/**************************************************************************
** defuze.me Epitech Innovative Project
**
** Copyright 2010-2011
**   Athena Calmettes - Jocelyn De La Rosa - Francois Gaillard
**   Adrien Jarthon - Alexandre Moore - Luc Peres - Arnaud Sellier
**
** All rights reserved.
**************************************************************************/

namespace Network {
	class Doorman;
}

#ifndef DOORMAN_HPP
#define DOORMAN_HPP

#include <QObject>
#include <QTcpServer>
#include <QTcpSocket>
#include "remoteevent.hpp"
#include "remotesock.hpp"
#include "dbcore.hpp"
#include "parameterizable.hpp"

namespace Network
{
	/** The doorman filter remote control connections.
	  * It uses white & black lists stored in the database, and ask the user for unknow devices.
	  * The devices will be identified by a unique key generated by the mobile application.
	  **/

	class Doorman : public Params::Parameterizable
	{
		Q_OBJECT
	public:
		Doorman(QTcpServer *rcServer);
		void				loadKnownDevices();
		void				storeKnownDevices();

	signals:
		void				authenticatedRemote(RemoteSock*);		///< emited when a client has successfully been authenticated
		void				needAuthorization(const RemoteEvent&);	///< emited when user's approval is needed

	public slots:
		void				newConnection();						///< called when a new client connects
		void				authentication(const RemoteEvent&);		///< called when an authentication event is received
		void				answered(RemoteSock*, unsigned, bool, bool);	///< called when the user answer through the dialog

	private:
		void				defineParams();							///< Load default params (called by parameterizable)
		QTcpServer			*rcServer;
		QVariantMap			knownDevices;
	};
}

#endif // DOORMAN_HPP
