#ifndef LIBRARYMODEL_HPP
#define LIBRARYMODEL_HPP

#include <QSqlQuery>
#include "audiotrack.hpp"
#include "treemodel.hpp"

namespace Library
{
	class LibraryPlugin;

	class LibraryModel : public TreeModel
	{
	public:

		enum SortKeys
		{
			INVALID, ALBUM, ARTIST, ALBUM_ARTIST, GENRE, TITLE, TRACK
		};
		enum SortKeysInfoData
		{
			TEXT = 0, FIELD = 1, ORDER_BY = 2, DEFAULT = 3
		};

		LibraryModel(LibraryPlugin *library);
		LibraryPlugin*					getLibrary() const;
		int								getIdByIndex(const QModelIndex &index) const;
		SortKeys						getKindByIndex(const QModelIndex &index) const;
		void							update(int sort = -1, QString search = QString());
		QStringList						mimeTypes() const;
		QMimeData*						mimeData(const QModelIndexList &indexes) const;
		Qt::ItemFlags					flags(const QModelIndex &index) const;
		QVariant						data(const QModelIndex &index, int role) const;

	protected:
		void							setupModelData();

	private:
		bool							isLowestLevelSortKey(SortKeys key);
		void							updateSortKeys(int sort);
		QString							updateSearch();
		QString 						orderByKeys() const;
		void							initKeys();
		void							buildQuery(QSqlQuery *query, int sort, const QString &search);
		void							buildTree(QSqlQuery *query, TreeItem *parent);
		void							clear();

		LibraryPlugin					*library;
		QHash<SortKeys, QStringList>	sortKeysInfo;
		QList<SortKeys>					sortKeys;
		QHash<QModelIndex, int>			idByIndexes;
		QHash<QModelIndex, SortKeys>	kindByIndexes;
		QString							currentSearch;
	};
}

#endif // LIBRARYMODEL_HPP
