/**************************************************************************
** defuze.me Epitech Innovative Project
**
** Copyright 2010-2011
**   Athena Calmettes - Jocelyn De La Rosa - Francois Gaillard
**   Adrien Jarthon - Alexandre Moore - Luc Peres - Arnaud Sellier
**
** All rights reserved.
**************************************************************************/

#ifndef MAINPLAYER_HPP
#define MAINPLAYER_HPP

#include "staticplugin.hpp"
#include "audiodecoder.hpp"
#include "playqueue.hpp"
#include "audiotrack.hpp"
#include "audioplayerwidget.hpp"
#include "audioplayer.hpp"
#include "audiomixer.hpp"
#include "audiooutputdevice.hpp"

namespace Player
{
	class MainPlayerWidget;
	class MainPlayer :public QObject, public StaticPlugin
	{
		Q_OBJECT
	public:
		MainPlayer(QString playerName);
		~MainPlayer();
		void		init();
		void		setPlayers(AudioPlayerWidget *A, AudioPlayerWidget *B);
		bool		play();
		bool		pause();
		bool		isPlaying() const;
		AudioPlayer*	firstPlayer();
		AudioPlayer*	secondPlayer();
	signals:
		void		played();
		void		paused();
		void		stopped();

		void		lplay(int uid);
		void		lpause(int uid, qint64 elapsedTime);
		void		lstop(int uid, qint64 elapsedTime);
		void		lnext(int oldUid, int newUid, qint64 elapsedTime);
	public slots:
		bool		crossfadeNext();
		bool		stop();
	private slots:
		void		crossfadeFinished();
		bool		loadTracks();
		bool		playA();
		bool		playB();
		void		newElem(Queue::Queueable*elem);
		void		removeElem(Queue::Queueable *elem);
		void		finished();
	private:
		void				emitPlayPause();

		AudioPlayer			*playingPlayer;
		AudioPlayerWidget	*playerWidgetA;
		AudioPlayerWidget	*playerWidgetB;

		AudioPlayer			*playerA;
		AudioPlayer			*playerB;
		Audio::AudioMixer	*mixerA;
		Audio::AudioMixer	*mixerB;
		Audio::AudioDecoder	*decoderA;
		Audio::AudioDecoder	*decoderB;

		QString				_playerName;

		Queue::PlayQueue	*playQueue;
		MainPlayerWidget	*widget;

		Gui::Module			*uiModule;

		Audio::AudioOutputDevice* outputDeviceA;
		Audio::AudioOutputDevice* outputDeviceB;

		bool				isCrossfading;
	};
}
#endif // MAINPLAYER_HPP
