/**************************************************************************
** defuze.me Epitech Innovative Project
**
** Copyright 2010-2012
**   Athena Calmettes - Jocelyn De La Rosa - Francois Gaillard
**   Adrien Jarthon - Alexandre Moore - Luc Peres - Arnaud Sellier
**
** All rights reserved.
**************************************************************************/

#include "editabletreeitem.hpp"
#include <QStringList>

EditableTreeItem::EditableTreeItem(const QVector<QVariant> &data, EditableTreeItem *parent)
{
	parentItem = parent;
	itemData = data;
}

EditableTreeItem::EditableTreeItem(const QString &title, EditableTreeItem *parent)
{
	parentItem = parent;
	QVector<QVariant> data;
	data << title;
	itemData = data;
}

EditableTreeItem::~EditableTreeItem()
{
	qDeleteAll(childItems);
}

EditableTreeItem *EditableTreeItem::child(int number)
{
	return childItems.value(number);
}

int EditableTreeItem::childCount() const
{
	return childItems.count();
}

int EditableTreeItem::childNumber() const
{
	if (parentItem)
		return parentItem->childItems.indexOf(const_cast<EditableTreeItem*>(this));

	return 0;
}

int EditableTreeItem::columnCount() const
{
	return itemData.count();
}

QVariant EditableTreeItem::data(int column) const
{
	return itemData.value(column);
}

bool EditableTreeItem::insertChildren(int position, int count, int columns)
{
	if (position < 0 || position > childItems.size())
		return false;

	for (int row = 0; row < count; ++row) {
		QVector<QVariant> data(columns);
		EditableTreeItem *item = new EditableTreeItem(data, this);
		childItems.insert(position, item);
	}

	return true;
}

bool EditableTreeItem::insertColumns(int position, int columns)
{
	if (position < 0 || position > itemData.size())
		return false;

	for (int column = 0; column < columns; ++column)
		itemData.insert(position, QVariant());

	foreach (EditableTreeItem *child, childItems)
		child->insertColumns(position, columns);

	return true;
}

EditableTreeItem *EditableTreeItem::parent()
{
	return parentItem;
}

bool EditableTreeItem::removeChildren(int position, int count)
{
	if (position < 0 || position + count > childItems.size())
		return false;

	for (int row = 0; row < count; ++row)
		delete childItems.takeAt(position);

	return true;
}

bool EditableTreeItem::removeColumns(int position, int columns)
{
	if (position < 0 || position + columns > itemData.size())
		return false;

	for (int column = 0; column < columns; ++column)
		itemData.remove(position);

	foreach (EditableTreeItem *child, childItems)
		child->removeColumns(position, columns);

	return true;
}

bool EditableTreeItem::setData(int column, const QVariant &value)
{
	if (column < 0 || column >= itemData.size())
		return false;

	itemData[column] = value;
	return true;
}
