/**************************************************************************
** defuze.me Epitech Innovative Project
**
** Copyright 2010-2011
**   Athena Calmettes - Jocelyn De La Rosa - Francois Gaillard
**   Adrien Jarthon - Alexandre Moore - Luc Peres - Arnaud Sellier
**
** All rights reserved.
**************************************************************************/

#include "clearablelineedit.hpp"

#include <QToolButton>
#include <QStyle>

ClearableLineEdit::ClearableLineEdit(QWidget *parent) : QLineEdit(parent)
{
	clearButton = new QToolButton(this);
	QPixmap pixmap(":/icons/clear");
	clearButton->setIcon(QIcon(pixmap));
	clearButton->setIconSize(pixmap.size());
	clearButton->setCursor(Qt::ArrowCursor);
	clearButton->setStyleSheet("QToolButton { border: none; padding: 0px; }");
	clearButton->hide();
	connect(clearButton, SIGNAL(clicked()), this, SLOT(clear()));
	connect(this, SIGNAL(textChanged(const QString&)), this, SLOT(updateCloseButton(const QString&)));
	int frameWidth = style()->pixelMetric(QStyle::PM_DefaultFrameWidth);
	setStyleSheet(QString("QLineEdit { border-radius: 10px; padding-left: 5px; padding-right: %1px; } ").arg(clearButton->sizeHint().width() + frameWidth + 1));
	QSize msz = minimumSizeHint();
	setMinimumSize(qMax(msz.width(), clearButton->sizeHint().height() + frameWidth * 2 + 2),
					qMax(msz.height(), clearButton->sizeHint().height() + frameWidth * 2 + 2));
}

void	ClearableLineEdit::resizeEvent(QResizeEvent *)
{
	QSize sz = clearButton->sizeHint();
	int frameWidth = style()->pixelMetric(QStyle::PM_DefaultFrameWidth);
	clearButton->move(rect().right() - frameWidth - sz.width() - 1,
						(rect().bottom() + 1 - sz.height())/2);
}

void ClearableLineEdit::updateCloseButton(const QString& text)
{
	clearButton->setVisible(!text.isEmpty());
}
